Package: warp
Title: Group Dates
Version: 0.2.3
Authors@R: c(
    person("Davis", "Vaughan", , "davis@posit.co", role = c("aut", "cre")),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: Tooling to group dates by a variety of periods including:
    yearly, monthly, by second, by week of the month, and more.  The
    groups are defined in such a way that they also represent the distance
    between dates in terms of the period. This extracts valuable
    information that can be used in further calculations that rely on a
    specific temporal spacing between observations.
License: MIT + file LICENSE
URL: https://github.com/DavisVaughan/warp,
        https://davisvaughan.github.io/warp/
BugReports: https://github.com/DavisVaughan/warp/issues
Depends: R (>= 4.0.0)
Suggests: covr, knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/build/compilation-database: true
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Config/usethis/last-upkeep: 2025-11-13
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-01-13 02:10:17 UTC; davis
Author: Davis Vaughan [aut, cre],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Davis Vaughan <davis@posit.co>
Repository: CRAN
Date/Publication: 2026-01-13 06:10:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 09:34:19 UTC; unix
Archs: warp.so.dSYM
