# Date + non-UTC origin converts with a warning

    Code
      out <- warp_distance(x, "year", origin = origin)
    Condition
      Warning:
      `x` (UTC) and `origin` (America/New_York) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "month", origin = origin)
    Condition
      Warning:
      `x` (UTC) and `origin` (America/New_York) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "yweek", origin = origin)
    Condition
      Warning:
      `x` (UTC) and `origin` (America/New_York) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "mday", origin = origin)
    Condition
      Warning:
      `x` (UTC) and `origin` (America/New_York) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "day", origin = origin)
    Condition
      Warning:
      `x` (UTC) and `origin` (America/New_York) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "hour", origin = origin)
    Condition
      Warning:
      `x` (UTC) and `origin` (America/New_York) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "minute", origin = origin)
    Condition
      Warning:
      `x` (UTC) and `origin` (America/New_York) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "second", origin = origin)
    Condition
      Warning:
      `x` (UTC) and `origin` (America/New_York) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "millisecond", origin = origin)
    Condition
      Warning:
      `x` (UTC) and `origin` (America/New_York) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

# UTC POSIXct + non-UTC origin converts with a warning

    Code
      out <- warp_distance(x, "year", origin = origin)
    Condition
      Warning:
      `x` (UTC) and `origin` (America/New_York) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "month", origin = origin)
    Condition
      Warning:
      `x` (UTC) and `origin` (America/New_York) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "yweek", origin = origin)
    Condition
      Warning:
      `x` (UTC) and `origin` (America/New_York) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "mday", origin = origin)
    Condition
      Warning:
      `x` (UTC) and `origin` (America/New_York) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "day", origin = origin)
    Condition
      Warning:
      `x` (UTC) and `origin` (America/New_York) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "hour", origin = origin)
    Condition
      Warning:
      `x` (UTC) and `origin` (America/New_York) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "minute", origin = origin)
    Condition
      Warning:
      `x` (UTC) and `origin` (America/New_York) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "second", origin = origin)
    Condition
      Warning:
      `x` (UTC) and `origin` (America/New_York) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "millisecond", origin = origin)
    Condition
      Warning:
      `x` (UTC) and `origin` (America/New_York) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

# local time POSIXct + UTC origin converts with a warning

    Code
      out <- warp_distance(x, "year", origin = origin)
    Condition
      Warning:
      `x` (local) and `origin` (UTC) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "month", origin = origin)
    Condition
      Warning:
      `x` (local) and `origin` (UTC) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "yweek", origin = origin)
    Condition
      Warning:
      `x` (local) and `origin` (UTC) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "mday", origin = origin)
    Condition
      Warning:
      `x` (local) and `origin` (UTC) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "day", origin = origin)
    Condition
      Warning:
      `x` (local) and `origin` (UTC) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "hour", origin = origin)
    Condition
      Warning:
      `x` (local) and `origin` (UTC) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "minute", origin = origin)
    Condition
      Warning:
      `x` (local) and `origin` (UTC) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "second", origin = origin)
    Condition
      Warning:
      `x` (local) and `origin` (UTC) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

---

    Code
      out <- warp_distance(x, "millisecond", origin = origin)
    Condition
      Warning:
      `x` (local) and `origin` (UTC) do not have the same time zone. Converting `x` to the time zone of `origin`. It is highly advised to provide `x` and `origin` with the same time zone.

